import React, {
  useState, useEffect, CSSProperties, useMemo,
} from 'react';
import {
  TUIGlobal,
  TUIStore,
  StoreName,
  NAME,
  CallStatus,
  TUICallKitServer,
  CallMediaType,
  VideoDisplayMode,
  VideoResolution,
} from '../../TUICallService';
import SingleCall from '../components/SingleCall/SingleCall';
import GroupCall from '../components/GroupCall/GroupCall';
import { classNames } from '../util/classnames';
import { CallInfoContext, UserInfoContext, CallerUserInfoContext } from '../context';
import { MessagePlugin } from '../components/base/Message/MessagePlugin';
import FloatingWindow from '../components/common/FloatingWindow';
import useGetSpeakers from '../hooks/useGetSpeakers';
import './TUICallKit.less';

interface ITUICallKitProps {
  style?: CSSProperties;
  className?: string;
  allowedMinimized?: boolean;
  allowedFullScreen?: boolean;
  videoDisplayMode?: VideoDisplayMode;
  videoResolution?: VideoResolution;
  beforeCalling?: any;
  afterCalling?: any;
}

export default function TUICallKit(props: ITUICallKitProps) {
  const {
    style,
    className,
    allowedMinimized = true,
    allowedFullScreen = true,
    videoDisplayMode = VideoDisplayMode.COVER,
    videoResolution = VideoResolution.RESOLUTION_1080P,
    beforeCalling,
    afterCalling,
  } = props;
  const [callStatus, setCallStatus] = useState(TUIStore.getData(StoreName.CALL, NAME.CALL_STATUS));
  const [isGroupCall, setIsGroupCall] = useState(TUIStore.getData(StoreName.CALL, NAME.IS_GROUP));
  const [
    callerUserInfo,
    setCallerUserInfo,
  ] = useState(TUIStore.getData(StoreName.CALL, NAME.CALLER_USER_INFO));
  const [
    localUserInfo,
    setLocalUserInfo,
  ] = useState(TUIStore.getData(StoreName.CALL, NAME.LOCAL_USER_INFO));
  const [
    remoteList,
    setRemoteList,
  ] = useState(TUIStore.getData(StoreName.CALL, NAME.REMOTE_USER_INFO_LIST));
  const [callType, setCallType] = useState(TUIStore.getData(StoreName.CALL, NAME.CALL_MEDIA_TYPE));
  const [callRole, setCallRole] = useState(TUIStore.getData(StoreName.CALL, NAME.CALL_ROLE));
  const [isEarPhone, setIsEarPhone] = useState(TUIStore.getData(StoreName.CALL, NAME.IS_EAR_PHONE));
  const [isFloat, setIsFloat] = useState(false);
  const speakerIdList = useGetSpeakers();
  const callInfoContextValue = useMemo(() => ({
    callStatus,
    callRole,
    callType,
    isEarPhone,
    isGroupCall,
    isFloat,
    allowedMinimized,
    allowedFullScreen,
  }), [callStatus, callRole, callType, isEarPhone, isGroupCall, isFloat, allowedFullScreen]);
  const UserInfoContextValue = useMemo(() => ({
    localUserInfo,
    remoteList,
  }), [localUserInfo, remoteList]);
  const classnames = classNames([
    className,
    'tuicallkit-container',
    {
      mobile: !TUIGlobal.isPC,
      float: isFloat,
    },
  ]);

  const handleCallStatusChange = (value) => {
    setCallStatus(value);
  };
  const handleIsGroupChange = (value) => {
    setIsGroupCall(value);
  };
  const handleCallerUserInfoChange = (value) => {
    setCallerUserInfo(value);
  };
  const handleLocalUserInfoChange = (value) => {
    setLocalUserInfo(value);
  };
  const handleRemoteUserInfoListChange = (value) => {
    setRemoteList(value);
  };
  const handleCallMediaTypeChange = (value) => {
    setCallType(value);
  };
  const handleCallRoleChange = (value) => {
    setCallRole(value);
  };
  const handEarPhoneChange = (value) => {
    setIsEarPhone(value);
  };

  const handleToastInfoChange = (value) => {
    if (value.text) {
      MessagePlugin[value.type || 'info']({ content: value.text });
    }
  };
  const handleIsMinizedChange = (value) => {
    setIsFloat(value);
  };

  const watchOptions = {
    [NAME.CALL_STATUS]: handleCallStatusChange,
    [NAME.IS_GROUP]: handleIsGroupChange,
    [NAME.CALLER_USER_INFO]: handleCallerUserInfoChange,
    [NAME.LOCAL_USER_INFO_EXCLUDE_VOLUMN]: handleLocalUserInfoChange,
    [NAME.REMOTE_USER_INFO_EXCLUDE_VOLUMN_LIST]: handleRemoteUserInfoListChange,
    [NAME.CALL_MEDIA_TYPE]: handleCallMediaTypeChange,
    [NAME.CALL_ROLE]: handleCallRoleChange,
    [NAME.IS_EAR_PHONE]: handEarPhoneChange,
    [NAME.TOAST_INFO]: handleToastInfoChange,
  };

  useEffect(() => {
    TUIStore.watch(
      StoreName.CALL,
      watchOptions,
      {
        notifyRangeWhenWatch: NAME.MYSELF,
      },
    );

    TUIStore.watch(StoreName.CALL, {
      [NAME.IS_MINIMIZED]: handleIsMinizedChange,
    });

    return () => {
      TUIStore.unwatch(
        StoreName.CALL,
        { ...watchOptions, [NAME.IS_MINIMIZED]: handleIsMinizedChange },
      );
    };
  }, []);

  useEffect(() => {
    TUICallKitServer.enableFloatWindow(allowedMinimized);
    TUICallKitServer.setVideoDisplayMode(videoDisplayMode as any);
    TUICallKitServer.setVideoResolution(videoResolution as any);
    TUICallKitServer.setCallback({
      beforeCalling,
      afterCalling,
    });
  }, [beforeCalling, afterCalling, TUICallKitServer, videoDisplayMode, videoResolution, allowedMinimized]);

  return callStatus !== CallStatus.IDLE && (
    <UserInfoContext.Provider value={UserInfoContextValue}>
      <CallerUserInfoContext.Provider value={callerUserInfo}>
        <CallInfoContext.Provider value={callInfoContextValue}>
          <div style={{ ...style }} className={classnames} id='tuicallkit-id'>
            {!isGroupCall && <SingleCall />}
            {isGroupCall && <GroupCall />}
          </div>
          {isFloat && (
            <FloatingWindow
              hasVideo={isGroupCall ? speakerIdList.length !== 0 : callType === CallMediaType.VIDEO}
              isAudioAvailable={localUserInfo.isAudioAvailable}
            />
          )}
        </CallInfoContext.Provider>
      </CallerUserInfoContext.Provider>
    </UserInfoContext.Provider>
  );
}
