import React, { useCallback, useContext, useMemo } from 'react';
import { CallMediaType, CallStatus, TUIGlobal } from '../../../../TUICallService';
import GridLayout from '../../base/GridLayout/GridLayout';
import { GridPlusItem } from '../../base/GridLayout';
import Portal from '../../base/Portal/Portal';
import Pusher from '../../common/Pusher/Pusher';
import { CallInfoContext, FocusItemContext, UserInfoContext } from '../../../context';
import { classNames } from '../../../util/classnames';
import useGetSpeakers from '../../../hooks/useGetSpeakers';
import Player from '../../common/Player/Player';
import Loading from '../../base/Loading';
import OverlayStream from '../../common/OverlayStream/OverlayStream';
import defaultAvatarSrc from '../../../assets/common/defaultAvatar.svg';
import './MediaContainer.less';

interface IRenderAudioStreamProps {
  avatar?: string;
  username?: string;
  visible?: boolean;
  isSmall?: boolean;
  isVideoAvailable?: boolean;
}

export default function MediaContainer() {
  const { localUserInfo, remoteList } = useContext(UserInfoContext);
  const { callStatus, callType, callRole } = useContext(CallInfoContext);
  const { focusItem, setFocusItem } = useContext(FocusItemContext);
  const { isFloat } = useContext(CallInfoContext);
  const speakerIdList = useGetSpeakers();

  const pusherLoading = useMemo(() => {
    if (callStatus === CallStatus.CALLING) {
      if ((callType === CallMediaType.AUDIO && !localUserInfo.isAudioAvailable)
          || (callType === CallMediaType.VIDEO && !localUserInfo.isVideoAvailable)) {
        return true;
      }
    }

    return false;
  }, [localUserInfo, callStatus, callType]);
  const renderLoading = useCallback((props) => {
    const { avatar } = props || {};

    return (
      <Loading theme='dot' style={{ zIndex: 1, position: 'absolute', top: 0 }}>
        <img alt='bg' style={{ width: '100%', height: '100%', objectFit: 'cover' }} src={avatar || defaultAvatarSrc} />
      </Loading>
    );
  }, [TUIGlobal.isPC, defaultAvatarSrc, callRole]);
  const renderAudioStream = (props: IRenderAudioStreamProps) => {
    const { avatar, username, visible } = props || {};

    return (
      <OverlayStream
        showLoading={false}
        showAvatar={false}
        avatar={avatar || defaultAvatarSrc}
        username={username}
        showUsername={TUIGlobal.isPC}
        showBackground
        isMobile={!TUIGlobal.isPC}
        showOverlayer={false}
        blur={false}
        style={{ visibility: visible ? 'visible' : 'hidden', zIndex: 1 }}
      />
    );
  };

  const renderRemoteList = useCallback(() => remoteList.map((stream, index) => (
    <GridPlusItem index={index + 1} key={stream.userId}>
      <Player
        className={classNames({ hidden: !TUIGlobal.isPC && !speakerIdList.includes(stream.domId) })}
        renderLoading={() => renderLoading({
          avatar: stream.avatar,
          username: stream.displayUserInfo,
        })}
        renderAudioStream={() => renderAudioStream({
          avatar: stream.avatar,
          username: stream.displayUserInfo,
          visible: !stream.isVideoAvailable,
        })}
        loading={!stream.isEnter}
        streamInfo={stream}
        showStreamInfo={!isFloat}
        isSpkear={speakerIdList.includes(stream.domId)}
      />
    </GridPlusItem>
  )), [remoteList, speakerIdList]);

  const classnames = classNames([
    'groupcall-gridplus-container',
    {
      pc: TUIGlobal.isPC,
      mobile: !TUIGlobal.isPC,
      float: isFloat,
      'two-layout': remoteList.length === 1 && focusItem === null,
    },
  ]);

  return (
    <div
      id='group-container'
      className={classNames([
        'stream-container',
        {
          mobile: !TUIGlobal.isPC,
          pc: TUIGlobal.isPC,
          float: isFloat,
        },
      ])}
    >
      <Portal attach={isFloat ? '#float' : '#group-container'} style={{ width: '100%', height: '100%', position: 'relative' }}>
        <GridLayout
          className={classnames}
          isMobile={!TUIGlobal.isPC}
          enableFocus={!TUIGlobal.isPC && !isFloat}
          unit={TUIGlobal.isPC ? '%' : 'vw'}
          onHandleFocusChange={setFocusItem}
        >
          <GridPlusItem index={0}>
            <Pusher
              className={classNames({
                hidden: !TUIGlobal.isPC && !speakerIdList.includes(localUserInfo.domId),
              })}
              callType={callType}
              loading={pusherLoading}
              showStreamInfo={!isFloat}
              renderLoading={() => renderLoading({
                avatar: localUserInfo.avatar,
                username: localUserInfo.userId,
              })}
              renderAudioStream={() => renderAudioStream({
                avatar: localUserInfo.avatar,
                username: localUserInfo.userId,
                visible: !localUserInfo.isVideoAvailable,
              })}
              isSpkear={speakerIdList.includes(localUserInfo.domId)}
            />
          </GridPlusItem>
          {...renderRemoteList()}
        </GridLayout>
      </Portal>
    </div>
  );
}
