import React, {
  useContext, useEffect, useMemo, useState,
} from 'react';
import { CallMediaType, CallStatus, TUIGlobal } from '../../../../TUICallService';
import { ToggleWindow, ToggleWindowItem } from '../../base/ToggleWindow';
import Player from '../../common/Player/Player';
import Pusher from '../../common/Pusher/Pusher';
import { CallInfoContext, UserInfoContext } from '../../../context';
import OverlayStream from '../../common/OverlayStream/OverlayStream';
import { classNames } from '../../../util/classnames';
import Portal from '../../base/Portal/Portal';
import defaultAvatarSrc from '../../../assets/common/defaultAvatar.svg';
import './MediaContainer.less';

interface IRenderAudioStreamProps {
  avatar?: string;
  username?: string;
  visible?: boolean;
  isSmall?: boolean;
  isVideoAvailable?: boolean;
}

export default function MediaContainer() {
  const { remoteList, localUserInfo } = useContext(UserInfoContext);
  const { callType, callStatus, isFloat } = useContext(CallInfoContext);
  const [currentFocus, setCurrentFocus] = useState('local');
  const isAudioCall = useMemo(() => callType === CallMediaType.AUDIO, [callType]);

  useEffect(() => {
    if (callStatus === CallStatus.CONNECTED) {
      setCurrentFocus('remote');
    }
  }, [callStatus]);

  const showSmallWindow = useMemo(() => {
    if (remoteList?.[0]?.isEnter && !isAudioCall) {
      return true;
    }
    return false;
  }, [remoteList, isAudioCall]);

  const renderAudioStream = (props: IRenderAudioStreamProps) => {
    let {
      avatar,
      username,
      isSmall,
      isVideoAvailable,
      ...rest
    } = props;
    let visibility: 'hidden' | 'visible' = 'hidden';

    if (!isFloat && (isAudioCall || !isVideoAvailable)) {
      visibility = 'visible';
    }

    if (isAudioCall) {
      avatar = remoteList[0].avatar;
      username = remoteList[0].userId;
    }

    return (
      <OverlayStream
        className='single-call-audio-stream'
        showLoading={false}
        avatar={avatar || defaultAvatarSrc}
        showAvatar={!TUIGlobal.isPC}
        username={username}
        showVolumn={TUIGlobal.isPC}
        isMobile={!TUIGlobal.isPC}
        isSmall={isSmall}
        style={{ visibility, zIndex: 1 }}
        {...rest}
      />
    );
  };

  const attachId = useMemo(() => {
    if (isFloat && !TUIGlobal.isPC) {
      return '#float';
    }

    return '#single-container';
  }, [isFloat]);
  const classnames = classNames([
    'single-stream-container',
    {
      mobile: !TUIGlobal.isPC,
      pc: TUIGlobal.isPC,
      float: isFloat,
    },
  ]);

  return (
    <div id='single-container' className={classnames}>
      <Portal attach={attachId} style={{ width: '100%', height: '100%', position: 'relative' }}>
        <ToggleWindow
          isMobile={!TUIGlobal.isPC}
          focus={currentFocus}
          showSmallWindow={showSmallWindow}
          onToggle={setCurrentFocus}
        >
          <ToggleWindowItem key='local' value='local'>
            <Pusher
              loading={
                callStatus === CallStatus.CALLING
                && (callType === CallMediaType.VIDEO ? !localUserInfo?.isVideoAvailable : true)
              }
              callType={callType}
              isSmall={currentFocus !== 'local'}
              renderAudioStream={() => renderAudioStream({
                isVideoAvailable: localUserInfo.isVideoAvailable,
                avatar: localUserInfo.avatar,
                username: localUserInfo.userId,
                isSmall: currentFocus !== 'local',
              })}
              callStatus={callStatus}
              showStreamInfo={TUIGlobal.isPC && callStatus === CallStatus.CONNECTED && !isAudioCall}
            />
          </ToggleWindowItem>
          <ToggleWindowItem key="remote" value='remote' className='remote'>
            {remoteList.length !== 0 && callStatus === CallStatus.CONNECTED && (
              <Player
                renderAudioStream={(props) => renderAudioStream({
                  isVideoAvailable: remoteList[0].isVideoAvailable,
                  avatar: remoteList[0].avatar,
                  username: remoteList[0].userId,
                  isSmall: currentFocus !== 'remote',
                  ...props,
                })}
                showStreamInfo={TUIGlobal.isPC && !isAudioCall}
                streamInfo={remoteList[0]}
              />
            )}
          </ToggleWindowItem>
        </ToggleWindow>
      </Portal>
    </div>
  );
}
