import React, {
  MutableRefObject,
  useEffect,
  useRef,
  useState,
  CSSProperties,
} from 'react';

const EventsFor = {
  touch: {
    start: 'touchstart',
    move: 'touchmove',
    stop: 'touchend',
  },
  mouse: {
    start: 'mousedown',
    move: 'mousemove',
    stop: 'mouseup',
  },
};

interface IDragableProps {
  nodeRef?: MutableRefObject<HTMLElement>;
  disable?: boolean;
  children?: any;
}

export function getTouchIdentifier(e) {
  if (e.targetTouches && e.targetTouches[0]) return e.targetTouches[0].identifier;
  if (e.changedTouches && e.changedTouches[0]) return e.changedTouches[0].identifier;
}

export function getTouch(e, identifier) {
  return (
    (e.targetTouches
      && Array.prototype.find.call(e.changedTouches, (t) => identifier === t.identifier))
    || (e.changedTouches
      && Array.prototype.find.call(e.changedTouches, (t) => identifier === t.identifier)));
}

export function offsetXYFromParent(evt, offsetParent, scale) {
  const isBody = offsetParent === offsetParent.ownerDocument.body;
  const offsetParentRect = isBody ? { left: 0, top: 0 } : offsetParent.getBoundingClientRect();

  const x = (evt.clientX + offsetParent.scrollLeft - offsetParentRect.left) / scale;
  const y = (evt.clientY + offsetParent.scrollTop - offsetParentRect.top) / scale;

  return { x, y };
}

function getControlPosition(e, touchIdentifier, draggableCore) {
  const touchObj = typeof touchIdentifier === 'number' ? getTouch(e, touchIdentifier) : null;
  if (typeof touchIdentifier === 'number' && !touchObj) return null; // not the right touch
  const node = draggableCore;
  // User can provide an offsetParent if desired.
  const offsetParent = node.offsetParent || node.ownerDocument.body;
  return offsetXYFromParent(touchObj || e, offsetParent, 1);
}
function innerHeight(node) {
  let height = node.clientHeight;
  const computedStyle = node.ownerDocument.defaultView.getComputedStyle(node);
  height -= parseInt(computedStyle.paddingTop, 10);
  height -= parseInt(computedStyle.paddingBottom, 10);
  return height;
}

function innerWidth(node) {
  let width = node.clientWidth;
  const computedStyle = node.ownerDocument.defaultView.getComputedStyle(node);
  width -= parseInt(computedStyle.paddingLeft, 10);
  width -= parseInt(computedStyle.paddingRight, 10);
  return width;
}
export function outerHeight(node) {
  // This is deliberately excluding margin for our calculations, since we are using
  // offsetTop which is including margin. See getBoundPosition
  let height = node.clientHeight;
  const computedStyle = node.ownerDocument.defaultView.getComputedStyle(node);
  height += parseInt(computedStyle.borderTopWidth, 10);
  height += parseInt(computedStyle.borderBottomWidth, 10);
  return height;
}

export function outerWidth(node) {
  // This is deliberately excluding margin for our calculations, since we are using
  // offsetLeft which is including margin. See getBoundPosition
  let width = node.clientWidth;
  const computedStyle = node.ownerDocument.defaultView.getComputedStyle(node);
  width += parseInt(computedStyle.borderLeftWidth, 10);
  width += parseInt(computedStyle.borderRightWidth, 10);
  return width;
}
function getBoundPosition(bounds, x, y, nodeRef) {
  // If no bounds, short-circuit and move on

  // Clone new bounds
  const node = nodeRef.current;

  if (typeof bounds === 'string') {
    const { ownerDocument } = node;
    const ownerWindow = ownerDocument.defaultView;
    let boundNode;
    if (bounds === 'parent') {
      boundNode = node.parentNode;
    } else {
      boundNode = ownerDocument.querySelector(bounds);
    }
    if (!(boundNode instanceof ownerWindow.HTMLElement)) {
      throw new Error(`Bounds selector "${bounds}" could not find an element.`);
    }
    const boundNodeEl: HTMLElement = boundNode; // for Flow, can't seem to refine correctly
    const nodeStyle = ownerWindow.getComputedStyle(node);
    const boundNodeStyle = ownerWindow.getComputedStyle(boundNodeEl);
    // Compute bounds. This is a pain with padding and offsets but this gets it exactly right.
    bounds = {
      left: -node.offsetLeft
        + parseInt(boundNodeStyle.paddingLeft, 10) + parseInt(nodeStyle.marginLeft, 10),
      top: -node.offsetTop
        + parseInt(boundNodeStyle.paddingTop, 10) + parseInt(nodeStyle.marginTop, 10),
      right: innerWidth(boundNodeEl) - outerWidth(node) - node.offsetLeft
        + parseInt(boundNodeStyle.paddingRight, 10) - parseInt(nodeStyle.marginRight, 10),
      bottom: innerHeight(boundNodeEl) - outerHeight(node) - node.offsetTop
        + parseInt(boundNodeStyle.paddingBottom, 10) - parseInt(nodeStyle.marginBottom, 10),
    };
  }

  // Keep x and y below right and bottom limits...
  if (typeof (bounds.right) === 'number') x = Math.min(x, bounds.right);
  if (typeof (bounds.bottom) === 'number') y = Math.min(y, bounds.bottom);

  // But above left and top limits.
  if (typeof (bounds.left) === 'number') x = Math.max(x, bounds.left);
  if (typeof (bounds.top) === 'number') y = Math.max(y, bounds.top);

  return [x, y];
}

export function Drag(props: IDragableProps) {
  const { nodeRef, disable = false, children } = props;
  const dragEventFor = useRef(EventsFor.mouse);
  const touchIdentifier = useRef();
  const [style, setStyle] = useState<CSSProperties>();

  function handleDrag(e) {
    // Get the current drag point from the event. This is used as the offset.
    const position = getControlPosition(e, touchIdentifier.current, nodeRef?.current);
    if (position == null) return;
    // let {x, y} = position;
    const [x, y] = getBoundPosition('body', position.x, position.y, nodeRef );
    setStyle({ transform: `translate(${x}px, ${y}px)` });
  }

  function handleDragStop(e) {
    const position = getControlPosition(e, touchIdentifier.current, nodeRef.current);
    if (position == null) return;
    const boundNodeStyle = nodeRef.current.ownerDocument.defaultView.getComputedStyle(document.querySelector('body'));
    // let {x, y} = position;
    const boundNodeEl = document.querySelector('body');
    const nodeStyle = nodeRef.current.ownerDocument.defaultView.getComputedStyle(nodeRef.current);
    // Compute bounds. This is a pain with padding and offsets but this gets it exactly right.
    const bounds = {
      left: -nodeRef.current.offsetLeft
        + parseInt(boundNodeStyle.paddingLeft, 10)
        + parseInt(nodeStyle.marginLeft, 10),
      top: -nodeRef.current.offsetTop
        + parseInt(boundNodeStyle.paddingTop, 10)
        + parseInt(nodeStyle.marginTop, 10),
      right: innerWidth(boundNodeEl)
        - outerWidth(nodeRef.current)
        - nodeRef.current.offsetLeft
        + parseInt(boundNodeStyle.paddingRight, 10)
        - parseInt(nodeStyle.marginRight, 10),
      bottom: innerHeight(boundNodeEl) - outerHeight(nodeRef.current) - nodeRef.current.offsetTop
        + parseInt(boundNodeStyle.paddingBottom, 10) - parseInt(nodeStyle.marginBottom, 10),
    };
    let [x, y] = getBoundPosition('body', position.x, position.y, nodeRef);

    if (x < parseInt(boundNodeStyle.width, 10) / 2) {
      x = bounds.left;
    } else {
      x = bounds.right;
    }

    setStyle({ transform: `translate(${x}px, ${y}px)` });

    const thisNode = nodeRef?.current;
    // Reset the el.
    // dragging = fals`e;
    // lastX = NaN;
    // lastY = NaN;`

    if (thisNode) {
      thisNode.ownerDocument.removeEventListener(dragEventFor.current.move, handleDrag);
      thisNode.ownerDocument.removeEventListener(dragEventFor.current.stop, handleDragStop);
    }
  }

  function handleDragStart(e) {
    const thisNode = nodeRef?.current;
    if (!thisNode || !thisNode.ownerDocument || !thisNode.ownerDocument.body) {
      throw new Error('<DraggableCore> not mounted on DragStart!');
    }

    if (e.type === 'touchstart') e.preventDefault();

    const { ownerDocument } = thisNode;
    touchIdentifier.current = getTouchIdentifier(e);

    // Get the current drag point from the event. This is used as the offset.
    const position = getControlPosition(e, touchIdentifier.current, nodeRef.current);
    if (position == null) return; // not possible but satisfies flow
    // let {x, y} = position;
    getBoundPosition('body', position.x, position.y, nodeRef);
    ownerDocument.addEventListener(dragEventFor.current.move, handleDrag);
    ownerDocument.addEventListener(dragEventFor.current.stop, handleDragStop);
  }

  function onTouchStart(e) {
    // We're on a touch device now, so change the event handlers
    dragEventFor.current = EventsFor.touch;
    return handleDragStart(e);
  }

  useEffect(
    () => !disable
      && nodeRef?.current.addEventListener(EventsFor.touch.start, onTouchStart, { passive: false }),
    [nodeRef, disable],
  );

  return (
    <>
      {React.cloneElement(children, { style })}
    </>
  );
}
